using FitConnect.Services.Models;

namespace FitConnect.Models;

public class Submission {
    public string Id { get; set; }
    public string DestinationId { get; init; }
    public string? CaseId { get; set; }

    public List<Attachment> Attachments { get; set; } = new();

    public ServiceType ServiceType { get; init; }

    public Callback? Callback { get; set; }


    /// <summary>
    ///     Fachdaten
    /// </summary>
    public Data? Data { get; set; }

    public Metadata Metadata { get; set; }


    public bool IsSubmissionReadyToAdd(out string? error) {
        var innerError = "";
        if (string.IsNullOrEmpty(DestinationId)) innerError += "DestinationId is required\r\n";

        if (ServiceType.IsValid()) innerError += "ServiceType is invalid\r\n";

        if (string.IsNullOrWhiteSpace(innerError)) {
            error = null;
            return true;
        }

        error = innerError.Trim();
        return false;
    }

    public bool IsSubmissionReadyToSend() {
        return true;
    }

    public static explicit operator Submission(SubmissionForPickupDto dto) {
        return new Submission {
            Id = dto.SubmissionId,
            Callback = null,
            DestinationId = dto.DestinationId,
            ServiceType = null
        };
    }

    public static explicit operator Submission(SubmissionDto dto) {
        return new Submission {
            Id = dto.SubmissionId,
            Callback = (Callback)dto.Callback,
            DestinationId = dto.DestinationId,
            ServiceType = (ServiceType)dto.ServiceType
        };
    }

    public static explicit operator CreateSubmissionDto(Submission sub) {
        return new CreateSubmissionDto {
            AnnouncedAttachments = sub.Attachments.Select(a => a.Id).ToList(),
            DestinationId = sub.DestinationId,
            ServiceType = (ServiceTypeDto)sub.ServiceType,
            Callback = (CallbackDto)sub.Callback
        };
    }
}
